@@ -1,16 +1,16 @@
 
 /*
  *  agCgi.c
- *  $Id: 4c2c6233a5c8260899b7c2b5446bd78a09040304 $
+ *  $Id$
  *
- *  Time-stamp:        "2009-07-31 11:01:07 bkorb"
+ *  Time-stamp:        "2010-02-24 08:43:40 bkorb"
  *
  *  This is a CGI wrapper for AutoGen.  It will take POST-method
  *  name-value pairs and emit AutoGen definitions to a spawned
  *  AutoGen process.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -76,7 +76,6 @@ static char const zOops[] =
 "AutoGen form processing error:\n";
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static char*
 parseInput( char* pzSrc, int len );
 /* = = = END-STATIC-FORWARD = = = */
@@ -95,7 +94,7 @@ loadCgi( void )
     pzOopsPrefix = zOops;
     {
         int tmpfd;
-        pzTmpStderr = mkstempPat();
+        AGDUPSTR(pzTmpStderr, mkstempPat(), "temp stderr file");
         tmpfd = mkstemp( pzTmpStderr );
         if (tmpfd < 0)
             AG_ABEND(aprf("failed to create temp file from `%s'", pzTmpStderr));
