@@ -1,14 +1,14 @@
 
 /*
  *  agUtils.c
- *  $Id: c2b939af0e5d78a17956ed485b79d6d2be85100e $
+ *  $Id$
  *
- *  Time-stamp:        "2009-11-01 14:10:47 bkorb"
+ *  Time-stamp:        "2010-02-24 08:43:36 bkorb"
  *
  *  This is the main routine for autogen.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -25,7 +25,6 @@
  */
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static tCC*
 skipQuote( tCC* pzQte );
 /* = = = END-STATIC-FORWARD = = = */
@@ -57,7 +56,7 @@ strlcpy( char* dest, tCC* src, size_t n )
 #endif
 
 
-LOCAL char*
+LOCAL char *
 aprf( char const* pzFmt, ... )
 {
     char* pz;
@@ -76,10 +75,11 @@ aprf( char const* pzFmt, ... )
 }
 
 
-LOCAL char*
+LOCAL char *
 mkstempPat( void )
 {
     static char const * pzTmpDir = NULL;
+    static size_t tmpdirlen = 0;
 
     if (pzTmpDir == NULL) {
         static char const * const envlist[] = {
@@ -94,12 +94,17 @@ mkstempPat( void )
                 break;
             }
         }
+        tmpdirlen = strlen(pzTmpDir);
     }
 
     {
-        char* pzRes = aprf("%s/agtmp.XXXXXX", pzTmpDir);
+        static char const tmp_pat[] = "/agtmp.XXXXXX";
+        size_t len = 2 * (tmpdirlen + sizeof(tmp_pat));
+        char* pzRes = AGALOC(len, "stemp pattern");
         manageAllocatedData((void*)pzRes);
         TAGMEM( pzRes, "temp file template" );
+        memcpy(pzRes, pzTmpDir, tmpdirlen);
+        memcpy(pzRes + tmpdirlen, tmp_pat, sizeof(tmp_pat));
         return pzRes;
     }
 }
@@ -167,9 +172,7 @@ doOptions( int arg_ct, char** arg_vec )
          *  Point to the character after the last '/', or to the full
          *  definition file name, if there is no '/'.
          */
-        // coverity[dereference] -- invalid analysis -- "pz" not dereferenced.
-        if (pz++ == NULL)
-            pz = OPT_ARG( DEFINITIONS );
+        pz = (pz == NULL) ? OPT_ARG( DEFINITIONS ) : (pz + 1);
 
         /*
          *  IF input is from stdin, then use "stdin"
