@@ -1,14 +1,14 @@
 
 /*
  *  autogen.c
- *  $Id: 039dc46983b320ee9be6d2c102aaa3d4e350caee $
+ *  $Id$
  *
- *  Time-stamp:        "2009-09-27 12:00:32 bkorb"
+ *  Time-stamp:        "2010-02-24 08:43:36 bkorb"
  *
  *  This is the main routine for autogen.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -37,7 +37,6 @@ typedef void (sighandler_proc_t)( int sig );
 static sighandler_proc_t ignoreSignal, abendSignal;
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static void
 inner_main( int argc, char** argv );
 
@@ -321,38 +320,38 @@ doneCheck( void )
 
     /*
      *  IF we diverted stderr, then now is the time to copy the text to stdout.
+     *  This is done for CGI mode wherein we produce an error page in case of
+     *  an error, but otherwise discard stderr.
      */
-    if (pzTmpStderr == NULL) {
-#if GUILE_VERSION >= 107000
-        if (exit_code != EXIT_SUCCESS)
-            _exit( exit_code );
-#endif
-        return;
+    if (pzTmpStderr != NULL) {
+        do {
+            long pos = ftell( stderr );
+            char* pz;
+
+            /*
+             *  Don't bother with the overhead if there is no work to do.
+             */
+            if (pos <= 0)
+                break;
+            pz = AGALOC( pos, "stderr redirected text" );
+            rewind( stderr );
+            fread(  pz, (size_t)1, (size_t)pos, stderr );
+            fwrite( pz, (size_t)1, (size_t)pos, stdout );
+            AGFREE( pz );
+        } while (0);
+
+        fclose( stderr );
+        unlink( pzTmpStderr );
+        AGFREE( pzTmpStderr );
+        pzTmpStderr = NULL;
     }
 
-    do {
-        long pos = ftell( stderr );
-        char* pz;
-
-        /*
-         *  Don't bother with the overhead if there is no work to do.
-         */
-        if (pos <= 0)
-            break;
-        pz = AGALOC( pos, "stderr redirected text" );
-        rewind( stderr );
-        fread(  pz, (size_t)1, (size_t)pos, stderr );
-        fwrite( pz, (size_t)1, (size_t)pos, stdout );
-        AGFREE( pz );
-    } while (0);
-
-    fclose( stderr );
-    unlink( pzTmpStderr );
-    AGFREE( pzTmpStderr );
-    pzTmpStderr = NULL;
 #if GUILE_VERSION >= 107000
-    if (exit_code != EXIT_SUCCESS)
-        _exit( exit_code );
+    if (exit_code != EXIT_SUCCESS) {
+        fflush(stderr);
+        fflush(stdout);
+        _exit(exit_code);
+    }
 #endif
 }
 
