@@ -1,9 +1,8 @@
 /*
  *  defDirect.c
- *  $Id: defDirect.c,v 4.31 2009/07/31 18:45:17 bkorb Exp $
+ *  $Id$
  *
- *  Time-stamp:        "2009-07-31 11:07:48 bkorb"
- *  Last Committed:    $Date: 2009/07/31 18:45:17 $
+ *  Time-stamp:        "2010-02-24 08:43:08 bkorb"
  *
  *  This module processes definition file directives.
  *
@@ -15,7 +14,7 @@
  *    trail='\+\+\+ End of Directives'
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2009 by Bruce Korb - all rights reserved
+ *  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -979,10 +978,14 @@ doDir_shell( char* pzArg, char* pzScan )
      *  "file text" that is used for more definitions.
      */
     pzText = runShell( pzText );
-    if (  (pzText == NULL)
-       || (*pzText == NUL))
+    if (pzText == NULL)
         return pzScan;
 
+    if (*pzText == NUL) {
+        AGFREE(pzText);
+        return pzScan;
+    }
+
     /*
      *  Get the space for the output data and for context overhead.
      *  This is an extra allocation and copy, but easier than rewriting
