@@ -1,17 +1,71 @@
 /*
- *   Character mapping generated 05/24/08 10:46:00
+ *   Character mapping generated 03/03/10 11:25:08
  *
  *  This file contains the character classifications
  *  used by AutoGen and AutoOpts for identifying tokens.
+ *  This file is part of AutoGen.
+ *  AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+ *  AutoGen is free software: you can redistribute it and/or modify it
+ *  under the terms of the GNU General Public License as published by the
+ *  Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ *  AutoGen is distributed in the hope that it will be useful, but
+ *  WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ *  See the GNU General Public License for more details.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 #ifndef AG_CHAR_MAP_H_GUARD
 #define AG_CHAR_MAP_H_GUARD 1
 
-#ifdef __sun
-# include <inttypes.h>
-#else
-# include <stdint.h>
-#endif
+#ifdef HAVE_CONFIG_H
+# if defined(HAVE_INTTYPES_H)
+#  include <inttypes.h>
+# elif defined(HAVE_STDINT_H)
+#  include <stdint.h>
+
+# else
+#   ifndef HAVE_INT8_T
+        typedef signed char     int8_t;
+#   endif
+#   ifndef HAVE_UINT8_T
+        typedef unsigned char   uint8_t;
+#   endif
+#   ifndef HAVE_INT16_T
+        typedef signed short    int16_t;
+#   endif
+#   ifndef HAVE_UINT16_T
+        typedef unsigned short  uint16_t;
+#   endif
+#   ifndef HAVE_UINT_T
+        typedef unsigned int    uint_t;
+#   endif
+
+#   ifndef HAVE_INT32_T
+#    if SIZEOF_INT == 4
+        typedef signed int      int32_t;
+#    elif SIZEOF_LONG == 4
+        typedef signed long     int32_t;
+#    endif
+#   endif
+
+#   ifndef HAVE_UINT32_T
+#    if SIZEOF_INT == 4
+        typedef unsigned int    uint32_t;
+#    elif SIZEOF_LONG == 4
+        typedef unsigned long   uint32_t;
+#    endif
+#   endif
+# endif /* HAVE_*INT*_H header */
+
+#else /* not HAVE_CONFIG_H -- */
+# ifdef __sun
+#  include <inttypes.h>
+# else
+#  include <stdint.h>
+# endif
+#endif /* HAVE_CONFIG_H */
 
 #if 0 /* mapping specification source (from autogen.map) */
 // 
@@ -22,6 +76,22 @@
 // %comment
 //         This file contains the character classifications
 //         used by AutoGen and AutoOpts for identifying tokens.
+// 
+//         This file is part of AutoGen.
+//         AutoGen Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
+// 
+//         AutoGen is free software: you can redistribute it and/or modify it
+//         under the terms of the GNU General Public License as published by the
+//         Free Software Foundation, either version 3 of the License, or
+//         (at your option) any later version.
+// 
+//         AutoGen is distributed in the hope that it will be useful, but
+//         WITHOUT ANY WARRANTY; without even the implied warranty of
+//         MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+//         See the GNU General Public License for more details.
+// 
+//         You should have received a copy of the GNU General Public License along
+//         with this program.  If not, see <http://www.gnu.org/licenses/>.
 // %
 // 
 // lower-case      "a-z"
