@@ -1,7 +1,7 @@
 
 /*
- *  $Id: enumeration.c,v 4.22 2008/01/23 00:36:04 bkorb Exp $
- * Time-stamp:      "2007-11-12 21:42:14 bkorb"
+ *  $Id$
+ * Time-stamp:      "2010-02-25 15:58:39 bkorb"
  *
  *   Automated Options Paged Usage module.
  *
@@ -10,8 +10,7 @@
  *
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is copyright (c) 1992-2008 by Bruce Korb - all rights reserved
- *  AutoOpts is copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -25,15 +24,14 @@
  *
  *  These files have the following md5sums:
  *
- *  239588c55c22c60ffe159946a760a33e pkg/libopts/COPYING.gplv3
- *  fa82ca978890795162346e661b47161a pkg/libopts/COPYING.lgplv3
+ *  43b91e8ca915626ed3818ffb1b71248b pkg/libopts/COPYING.gplv3
+ *  06a1a2e4760c90ea5e1dad8dfaac4d39 pkg/libopts/COPYING.lgplv3
  *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
  */
 
 tSCC*  pz_enum_err_fmt;
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static void
 enumError(
     tOptions*     pOpts,
@@ -57,15 +55,15 @@ enumError(
     tCC* const *  paz_names,
     int           name_ct )
 {
-    static char const mask_msg[] =
-        "  or an integer mask with any of the lower %d bits set\n";
-
     size_t max_len = 0;
     size_t ttl_len = 0;
     int    ct_down = name_ct;
     int    hidden  = 0;
 
-    if (pOpts > (tOptions*)0x0FUL)
+    /*
+     *  A real "pOpts" pointer means someone messed up.  Give a real error.
+     */
+    if (pOpts > OPTPROC_EMIT_LIMIT)
         fprintf(option_usage_fp, pz_enum_err_fmt, pOpts->pzProgName,
                 pOD->optArg.argString, pOD->pz_Name);
 
@@ -149,21 +147,20 @@ enumError(
         }
         fprintf(option_usage_fp, "%s\n", *paz_names);
     }
-    if (pOpts == (tOptions*)0x01UL)
-        fprintf(option_usage_fp, mask_msg, name_ct);
-    else
-        fprintf(option_usage_fp, "  or an integer between %d and %d\n",
-                hidden, name_ct - 1 + hidden);
 
-    /*
-     *  IF we do not have a pOpts pointer, then this output is being requested
-     *  by the usage procedure.  Let's not re-invoke it recursively.
-     */
-    if (pOpts > (tOptions*)0x0FUL)
+    if (pOpts > OPTPROC_EMIT_LIMIT) {
+        fprintf(option_usage_fp, zIntRange, hidden, name_ct - 1 + hidden);
+
         (*(pOpts->pUsageProc))( pOpts, EXIT_FAILURE );
+        /* NOTREACHED */
+    }
 
-    if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_MEMBERSHIP)
+    if (OPTST_GET_ARGTYPE(pOD->fOptState) == OPARG_TYPE_MEMBERSHIP) {
+        fprintf(option_usage_fp, zLowerBits, name_ct);
         fputs(zSetMemberSettings, option_usage_fp);
+    } else {
+        fprintf(option_usage_fp, zIntRange, hidden, name_ct - 1 + hidden);
+    }
 }
 
 
@@ -208,20 +205,10 @@ findName(
     if (res < name_ct)
         return res; /* partial match */
 
-    if (res == name_ct) {
-        pz_enum_err_fmt = zNoKey; /* no partial matches */
-
-    } else {
-        /*
-         *  Too many partial matches.
-         */
-        pz_enum_err_fmt = zAmbigKey;
-        res = name_ct;
-    }
-
+    pz_enum_err_fmt = (res == name_ct) ? zNoKey : zAmbigKey;
     option_usage_fp = stderr;
     enumError(pOpts, pOD, paz_names, (int)name_ct);
-    return res;
+    return name_ct;
 }
 
 
@@ -245,7 +232,7 @@ optionKeywordName(
     tOptDesc od;
 
     od.optArg.argEnum = enum_val;
-    (*(pOD->pOptProc))( (void*)(2UL), &od );
+    (*(pOD->pOptProc))(OPTPROC_RETURN_VALNAME, &od );
     return od.optArg.argString;
 }
 
@@ -282,14 +269,14 @@ optionEnumerationVal(
      *  then it is some sort of special request.
      */
     switch ((uintptr_t)pOpts) {
-    case 0UL:
+    case (uintptr_t)OPTPROC_EMIT_USAGE:
         /*
          *  print the list of enumeration names.
          */
         enumError(pOpts, pOD, paz_names, (int)name_ct);
         break;
 
-    case 1UL:
+    case (uintptr_t)OPTPROC_EMIT_SHELL:
     {
         unsigned int ix = pOD->optArg.argEnum;
         /*
@@ -303,7 +290,7 @@ optionEnumerationVal(
         break;
     }
 
-    case 2UL:
+    case (uintptr_t)OPTPROC_RETURN_VALNAME:
     {
         tSCC zInval[] = "*INVALID*";
         unsigned int ix = pOD->optArg.argEnum;
@@ -313,12 +300,12 @@ optionEnumerationVal(
         if (ix >= name_ct)
             return (uintptr_t)zInval;
 
-        res = (uintptr_t)paz_names[ ix ];
+        pOD->optArg.argString = paz_names[ix];
         break;
     }
 
     default:
-        res = findName( pOD->optArg.argString, pOpts, pOD, paz_names, name_ct );
+        res = findName(pOD->optArg.argString, pOpts, pOD, paz_names, name_ct);
 
         if (pOD->fOptState & OPTST_ALLOC_ARG) {
             AGFREE(pOD->optArg.argString);
@@ -359,39 +346,43 @@ optionSetMembers(
      *  then it is some sort of special request.
      */
     switch ((uintptr_t)pOpts) {
-    case 0UL:
+    case (uintptr_t)OPTPROC_EMIT_USAGE:
         /*
          *  print the list of enumeration names.
          */
-        enumError((tOptions*)0x01UL, pOD, paz_names, (int)name_ct );
+        enumError(OPTPROC_EMIT_USAGE, pOD, paz_names, (int)name_ct );
         return;
 
-    case 1UL:
+    case (uintptr_t)OPTPROC_EMIT_SHELL:
     {
         /*
          *  print the name string.
          */
+        int       ix   =  0;
         uintptr_t bits = (uintptr_t)pOD->optCookie;
-        uintptr_t res  = 0;
         size_t    len  = 0;
 
+        bits &= ((uintptr_t)1 << (uintptr_t)name_ct) - (uintptr_t)1;
+
         while (bits != 0) {
             if (bits & 1) {
                 if (len++ > 0) fputs( " | ", stdout );
-                fputs( paz_names[ res ], stdout );
+                fputs(paz_names[ix], stdout);
             }
-            if (++res >= name_ct) break;
+            if (++ix >= name_ct) break;
             bits >>= 1;
         }
         return;
     }
 
-    case 2UL:
+    case (uintptr_t)OPTPROC_RETURN_VALNAME:
     {
         char*     pz;
         uintptr_t bits = (uintptr_t)pOD->optCookie;
-        uintptr_t res  = 0;
-        size_t    len  = 0;
+        int       ix   = 0;
+        size_t    len  = 5;
+
+        bits &= ((uintptr_t)1 << (uintptr_t)name_ct) - (uintptr_t)1;
 
         /*
          *  Replace the enumeration value with the name string.
@@ -399,12 +390,12 @@ optionSetMembers(
          */
         while (bits != 0) {
             if (bits & 1)
-                len += strlen( paz_names[ res ]) + 8;
-            if (++res >= name_ct) break;
+                len += strlen( paz_names[ix]) + 8;
+            if (++ix >= name_ct) break;
             bits >>= 1;
         }
 
-        pOD->optArg.argString = pz = AGALOC( len, "enum name" );
+        pOD->optArg.argString = pz = AGALOC(len, "enum name");
 
         /*
          *  Start by clearing all the bits.  We want to turn off any defaults
@@ -414,14 +405,16 @@ optionSetMembers(
         strcpy( pz, "none" );
         pz += 4;
         bits = (uintptr_t)pOD->optCookie;
-        res = 0;
+        bits &= ((uintptr_t)1 << (uintptr_t)name_ct) - (uintptr_t)1;
+        ix = 0;
+
         while (bits != 0) {
             if (bits & 1) {
                 strcpy( pz, " + " );
-                strcpy( pz+3, paz_names[ res ]);
-                pz += strlen( paz_names[ res ]) + 3;
+                strcpy( pz+3, paz_names[ix]);
+                pz += strlen( paz_names[ix]) + 3;
             }
-            if (++res >= name_ct) break;
+            if (++ix >= name_ct) break;
             bits >>= 1;
         }
         return;
@@ -431,6 +424,9 @@ optionSetMembers(
         break;
     }
 
+    if ((pOD->fOptState & OPTST_RESET) != 0)
+        return;
+
     {
         tCC*      pzArg = pOD->optArg.argString;
         uintptr_t res;
@@ -469,6 +465,8 @@ optionSetMembers(
                 if (pz != pzArg + len) {
                     char z[ AO_NAME_SIZE ];
                     tCC* p;
+                    int  shift_ct;
+
                     if (*pz != NUL) {
                         if (len >= AO_NAME_LIMIT)
                             break;
@@ -479,7 +477,12 @@ optionSetMembers(
                         p = pzArg;
                     }
 
-                    bit = 1UL << findName(p, pOpts, pOD, paz_names, name_ct);
+                    shift_ct = findName(p, pOpts, pOD, paz_names, name_ct);
+                    if (shift_ct >= name_ct) {
+                        pOD->optCookie = (void*)0;
+                        return;
+                    }
+                    bit = 1UL << shift_ct;
                 }
                 if (iv)
                      res &= ~bit;
