@@ -1,7 +1,7 @@
 
 /*
- *  usage.c  $Id: usage.c,v 4.24 2008/01/23 00:36:05 bkorb Exp $
- * Time-stamp:      "2007-11-12 20:49:02 bkorb"
+ *  usage.c  $Id$
+ * Time-stamp:      "2010-02-24 08:39:39 bkorb"
  *
  *  This module implements the default usage procedure for
  *  Automated Options.  It may be overridden, of course.
@@ -15,8 +15,7 @@
 /*
  *  This file is part of AutoOpts, a companion to AutoGen.
  *  AutoOpts is free software.
- *  AutoOpts is copyright (c) 1992-2008 by Bruce Korb - all rights reserved
- *  AutoOpts is copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoOpts is Copyright (c) 1992-2010 by Bruce Korb - all rights reserved
  *
  *  AutoOpts is available under any one of two licenses.  The license
  *  in use must be one of these two and the choice is under the control
@@ -30,8 +29,8 @@
  *
  *  These files have the following md5sums:
  *
- *  239588c55c22c60ffe159946a760a33e pkg/libopts/COPYING.gplv3
- *  fa82ca978890795162346e661b47161a pkg/libopts/COPYING.lgplv3
+ *  43b91e8ca915626ed3818ffb1b71248b pkg/libopts/COPYING.gplv3
+ *  06a1a2e4760c90ea5e1dad8dfaac4d39 pkg/libopts/COPYING.lgplv3
  *  66a5cedaf62c4b2637025f049f9b826f pkg/libopts/COPYING.mbsd
  */
 
@@ -44,7 +43,6 @@ static char    zOptFmtLine[ 16 ];
 static ag_bool displayEnum;
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by mk-fwd */
 static ag_bool
 checkGNUUsage( tOptions* pOpts );
 
@@ -61,6 +59,12 @@ printInitList(
     tCC*        pzRc,
     tCC*        pzPN );
 
+static void
+printOptPreamble(
+    tOptions*     pOptions,
+    tOptDesc*     pOD,
+    arg_types_t*  pAT );
+
 static void
 printOneUsage(
     tOptions*     pOptions,
@@ -69,9 +73,9 @@ printOneUsage(
 
 static void
 printOptionUsage(
-    tOptions* pOpts,
-    int       ex_code,
-    tCC*      pOptTitle );
+    tOptions *  pOpts,
+    int         ex_code,
+    tCC *       pOptTitle );
 
 static void
 printProgramDetails( tOptions* pOptions );
@@ -331,13 +335,13 @@ printExtendedUsage(
          *  THEN call it, requesting the range or other special info
          */
         if (  (pOD->pOptProc != NULL)
-              && (pOD->pOptProc != optionNumericVal) ) {
-            (*(pOD->pOptProc))(pOptions, NULL);
+           && (pOD->pOptProc != optionNumericVal) ) {
+            (*(pOD->pOptProc))(OPTPROC_EMIT_USAGE, pOD);
         }
         break;
 
     case OPARG_TYPE_FILE:
-        (*(pOD->pOptProc))(pOptions, NULL);
+        (*(pOD->pOptProc))(OPTPROC_EMIT_USAGE, pOD);
         break;
     }
 
@@ -463,11 +467,8 @@ printInitList(
 }
 
 
-/*
- *  Print the usage information for a single option.
- */
 static void
-printOneUsage(
+printOptPreamble(
     tOptions*     pOptions,
     tOptDesc*     pOD,
     arg_types_t*  pAT )
@@ -480,17 +481,31 @@ printOneUsage(
      */
     if ((pOptions->fOptSet & OPTPROC_SHORTOPT) == 0)
         fputs( pAT->pzSpc, option_usage_fp );
+
     else if (! IS_GRAPHIC_CHAR(pOD->optValue)) {
         if (  (pOptions->fOptSet & (OPTPROC_GNUUSAGE|OPTPROC_LONGOPT))
            == (OPTPROC_GNUUSAGE|OPTPROC_LONGOPT))
             fputc( ' ', option_usage_fp );
         fputs( pAT->pzNoF, option_usage_fp );
+
     } else {
         fprintf( option_usage_fp, "   -%c", pOD->optValue );
         if (  (pOptions->fOptSet & (OPTPROC_GNUUSAGE|OPTPROC_LONGOPT))
            == (OPTPROC_GNUUSAGE|OPTPROC_LONGOPT))
             fputs( ", ", option_usage_fp );
     }
+}
+
+/*
+ *  Print the usage information for a single option.
+ */
+static void
+printOneUsage(
+    tOptions*     pOptions,
+    tOptDesc*     pOD,
+    arg_types_t*  pAT )
+{
+    printOptPreamble(pOptions, pOD, pAT);
 
     {
         char  z[ 80 ];
@@ -512,6 +527,7 @@ printOneUsage(
         case OPARG_TYPE_NUMERIC:     pzArgType = pAT->pzNum;  break;
         case OPARG_TYPE_HIERARCHY:   pzArgType = pAT->pzNest; break;
         case OPARG_TYPE_STRING:      pzArgType = pAT->pzStr;  break;
+        case OPARG_TYPE_TIME:        pzArgType = pAT->pzTime; break;
         default:                     goto bogus_desc;
         }
 
@@ -539,18 +555,36 @@ printOneUsage(
  */
 static void
 printOptionUsage(
-    tOptions* pOpts,
-    int       ex_code,
-    tCC*      pOptTitle )
+    tOptions *  pOpts,
+    int         ex_code,
+    tCC *       pOptTitle )
 {
-    int        ct     = pOpts->optCt;
-    int        optNo  = 0;
-    tOptDesc*  pOD    = pOpts->pOptDesc;
-    int        docCt  = 0;
+    int         ct     = pOpts->optCt;
+    int         optNo  = 0;
+    tOptDesc *  pOD    = pOpts->pOptDesc;
+    int         docCt  = 0;
 
     do  {
-        if ((pOD->fOptState & OPTST_OMITTED) != 0)
+        if ((pOD->fOptState & OPTST_NO_USAGE_MASK) != 0) {
+
+            /*
+             * IF      this is a compiled-out option
+             *   *AND* usage was requested with "omitted-usage"
+             *   *AND* this is NOT abbreviated usage
+             * THEN display this option.
+             */
+            if (  (pOD->fOptState == (OPTST_OMITTED | OPTST_NO_INIT))
+               && (pOD->pz_Name != NULL)
+               && (ex_code == EXIT_SUCCESS))  {
+
+                char const * why_pz =
+                    (pOD->pzText == NULL) ? zDisabledWhy : pOD->pzText;
+                printOptPreamble(pOpts, pOD, &argTypes);
+                fprintf(option_usage_fp, zDisabledOpt, pOD->pz_Name, why_pz);
+            }
+
             continue;
+        }
 
         if ((pOD->fOptState & OPTST_DOCUMENT) != 0) {
             if (ex_code == EXIT_SUCCESS) {
@@ -570,12 +604,12 @@ printOptionUsage(
          *  THEN document that the remaining options are not user opts
          */
         if (  (pOpts->presetOptCt == optNo)
-              && (ex_code == EXIT_SUCCESS)
-              && (docCt > 0)
-              && ((pOD[-1].fOptState & OPTST_DOCUMENT) == 0) )
+           && (ex_code == EXIT_SUCCESS)
+           && (docCt > 0)
+           && ((pOD[-1].fOptState & OPTST_DOCUMENT) == 0) )
             fprintf( option_usage_fp, argTypes.pzBrk, zAuto, pOptTitle );
 
-        printOneUsage( pOpts, pOD, &argTypes );
+        printOneUsage(pOpts, pOD, &argTypes);
 
         /*
          *  IF we were invoked because of the --help option,
@@ -631,7 +665,7 @@ printProgramDetails( tOptions* pOptions )
             switch (OPTST_GET_ARGTYPE(pOD->fOptState)) {
             case OPARG_TYPE_ENUMERATION:
             case OPARG_TYPE_MEMBERSHIP:
-                (*(pOD->pOptProc))( NULL, pOD );
+                (*(pOD->pOptProc))(OPTPROC_EMIT_USAGE, pOD);
             }
         }  while (pOD++, optNo++, (--ct > 0));
     }
@@ -669,6 +703,7 @@ setGnuOptFmts( tOptions* pOpts, tCC** ppT )
     argTypes.pzNum  = zGnuNumArg;
     argTypes.pzKey  = zGnuKeyArg;
     argTypes.pzKeyL = zGnuKeyLArg;
+    argTypes.pzTime = zGnuTimeArg;
     argTypes.pzFile = zGnuFileArg;
     argTypes.pzBool = zGnuBoolArg;
     argTypes.pzNest = zGnuNestArg;
@@ -707,6 +742,7 @@ setStdOptFmts( tOptions* pOpts, tCC** ppT )
     argTypes.pzNum  = zStdNumArg;
     argTypes.pzKey  = zStdKeyArg;
     argTypes.pzKeyL = zStdKeyLArg;
+    argTypes.pzTime = zStdTimeArg;
     argTypes.pzFile = zStdFileArg;
     argTypes.pzBool = zStdBoolArg;
     argTypes.pzNest = zStdNestArg;
